"""
{{ project_name | pascal_case }} Application Components module.

This module contains reusable widgets and components
Version: {{ version }}
"""

# Import your widgets here...
from .reactive_text import MyReactiveText

__all__ = [
    'MyReactiveText'
]