"""
Controllers package for {{ project_name | pascal_case }}.

Controllers contain the business logic and manage application state.
"""

from .counter import CounterController

__all__ = [
    'CounterController'
]